@@ -13,6 +13,10 @@ rare.status <- function(msg, verbose=TRUE){
 rtk <- function(input, repeats = 10, depth = 0, ReturnMatrix = 0, margin = 2, verbose = FALSE, threads = 1, tmpdir = NULL ){
 
 
+    # pass 1:x to Cpp as colnames
+    removeCnames <- FALSE
+    removeRnames <- FALSE
+
   if(repeats < ReturnMatrix ){
     repeats <- ReturnMatrix
     warning(paste("Repeats can not be smaller than number of matrices to return. Repeats set to match ReturnMatrix. repeats = ReturnMatrix =", repeats, sep=" "))
@@ -35,9 +39,7 @@ rtk <- function(input, repeats = 10, depth = 0, ReturnMatrix = 0, margin = 2, ve
       stop("The supplied matrix object is not numeric. Please check your input matrix.")
     }
 
-    # pass 1:x to Cpp as colnames
-    removeCnames <- FALSE
-    removeRnames <- FALSE
+
     if(is.null(colnames(input))){
       colnames(input) <- paste("col ", seq(1:ncol(input)), sep="")
       removeCnames <- TRUE
@@ -49,28 +51,15 @@ rtk <- function(input, repeats = 10, depth = 0, ReturnMatrix = 0, margin = 2, ve
 
     # call the actual software
 
-    result <- lapply(depth, function(d){
-            res <- rcpp_rarefaction("", input, colnames(input),
-                                    rownames(input), repeats, d,
+    result <-  rcpp_rarefaction("", input, colnames(input),
+                                    rownames(input), repeats, depth,
                                     ReturnMatrix, verbose, threads,
                                     margin, "NULL", FALSE)
-            gc()
+    gc()
 
-            # remove col and/or row names, as we've added them for
-            # the Cpp software to work well
-            if(removeRnames == TRUE && removeRnames == TRUE){
-              res$raremat <- lapply(res$raremat, unname)
-            }else{
-              if(removeRnames == TRUE){
-                res$raremat <- lapply(res$raremat, function(x) {rownames(x) <- NULL; return (x)})
-              }
-              if(removeCnames == TRUE){
-                res$raremat <- lapply(res$raremat, function(x) {colnames(x) <- NULL; return (x)})
-              }
-            }
-            gc()
-            return(res)
-          })
+    
+        
+          
 
 
   }else if(class(input) == "character"){
@@ -79,7 +68,7 @@ rtk <- function(input, repeats = 10, depth = 0, ReturnMatrix = 0, margin = 2, ve
       uselowmem <- TRUE;
       rare.status("Low memory mode will be used. Temporary files will be stored on storage medium", verbose)
     }else if(!is.null(tmpdir) & margin != 2){
-      warning("Can not use low mem on margin = 1. Please consider transforming your input data, to use low mem mode.")
+      warning("Can not use low mem on margin = 1. Please consider transforming your input data, to use low memory (swap) mode.")
     }else{
       uselowmem   <- FALSE;
       tmpdir     <- "NULL";
@@ -89,30 +78,41 @@ rtk <- function(input, repeats = 10, depth = 0, ReturnMatrix = 0, margin = 2, ve
     if(!file.exists(input)){
       stop(paste("The file can not be found. Please verify that the file exists in the given location. The path given is:", input, sep = " "))
     }
-    result <- lapply(depth, function(d){
-              res <- rcpp_rarefaction( input,
-                                       matrix(1,1,c(1)),
-                                       c(NA),c(NA), # col and rownames
-                                       repeats, d,
-                                       ReturnMatrix,
-                                       verbose, threads,
-                                       margin, tmpdir, uselowmem)
-              gc()
-              return(res)
-            })
+    result <- rcpp_rarefaction( input,
+                               matrix(1,1,c(1)),
+                               c(NA),c(NA), # col and rownames
+                               repeats, depth,
+                               ReturnMatrix,
+                               verbose, threads,
+                               margin, tmpdir, uselowmem)
+
+            
   }else{
     stop("Unknown input type. Path to a file (character) or a numeric matrix are accepted types.")
   }
     result <- lapply(result, function(res){
+        # remove names, if there werent any
+        if(removeRnames == TRUE && removeRnames == TRUE){
+          res$raremat <- lapply(res$raremat, unname)
+        }else{
+          if(removeRnames == TRUE){
+            res$raremat <- lapply(res$raremat, function(x) {rownames(x) <- NULL; return (x)})
+          }
+          if(removeCnames == TRUE){
+            res$raremat <- lapply(res$raremat, function(x) {colnames(x) <- NULL; return (x)})
+          }
+        }
+            
+        
 
-    if(length(res$skipped) > 0){
-      warning(paste(length(res$skipped), "samples where skipped because the depth was greater than the number of elements in the sample."))
-    }
+        if(length(res$skipped) > 0){
+          warning(paste(length(res$skipped), "samples where skipped because the depth was greater than the number of elements in the sample."))
+        }
 
-    # calculate median for diversity measures
-    measures               <- c('richness', 'shannon', 'simpson', 'invsimpson', 'chao1', 'eveness')
-    res$div.median         <- lapply(measures, r.median, x=res$divvs)
-    names(res$div.median)  <- paste("median.", measures, sep = "")
+        # calculate median for diversity measures
+        measures               <- c('richness', 'shannon', 'simpson', 'invsimpson', 'chao1', 'eveness')
+        res$div.median         <- lapply(measures, r.median, x=res$divvs)
+        names(res$div.median)  <- paste("median.", measures, sep = "")
 
     return(res)
   })
